use crate::Point;

// #[macro_export(local_inner_macros)]
macro_rules! point_from_string {
    ( $t:ty ) => {
        impl TryFrom<String> for Point<$t> {
            type Error = anyhow::Error;
            fn try_from(value: String) -> Result<Self, Self::Error> {
                if !value.contains("(") || !value.contains(")") || !value.contains(",") {
                    anyhow::bail!("malformed point, expected \"(x, y)\", got {:?}", value);
                }
                let mut work = value.clone().trim().to_string();
                work = work.replace("(", "").replace(")", "").replace(" ", "");
                if let Some(parts) = work.split_once(",") {
                    let r = parts.0.parse::<$t>();
                    match r {
                        Ok(x) => {
                            let r2 = parts.1.parse::<$t>();
                            match r2 {
                                Ok(y) => {
                                    return Ok(Point{x, y})
                                }
                                Err(e) => {
                                    return Err(anyhow::format_err!("malformed y value, {}", e));
                                }
                            }
                        },
                        Err(e) => {
                            return Err(anyhow::format_err!("malformed x value, {}", e));
                        }
                    }
                } else {
                    anyhow::bail!("malformed point, expected \"(x, y)\", got {:?}", value);
                }
            }
        }
    };
}

macro_rules! point_distance {
    ($t:ty) => {
        impl Point<$t> {
            #[allow(unused_assignments)]
            pub fn distance(&self, other: &Self) -> f64 {
                let mut x1: $t = 0;
                let mut x2: $t = 0;
                let mut y1: $t = 0;
                let mut y2: $t = 0;
                if self.x > other.x {
                    x1 = self.x;
                    x2 = other.x;
                } else {
                    x1 = other.x;
                    x2 = self.x;
                }
                if self.y > other.y {
                    y1 = self.y;
                    y2 = other.y;
                } else {
                    y1 = other.y;
                    y2 = self.y;
                }
                let dx = x1 - x2;
                let dy = y1 - y2;
                let a = (dx * dx) as f64;
                let b = (dy * dy) as f64;
                (a + b).sqrt()
            }
        }
    };
}

// Usage

// impl TryFrom<String> for Point<Integer>
point_from_string!(i8);
point_from_string!(i16);
point_from_string!(i32);
point_from_string!(i64);
point_from_string!(i128);
point_from_string!(u8);
point_from_string!(u16);
point_from_string!(u32);
point_from_string!(u64);
point_from_string!(u128);

// Define distance for Point, but handles unsigned integers (that don't have Neg)
point_distance!(i8);
point_distance!(i16);
point_distance!(i32);
point_distance!(i64);
point_distance!(i128);
point_distance!(u8);
point_distance!(u16);
point_distance!(u32);
point_distance!(u64);
point_distance!(u128);
